package {

/**
 *	Class description.
 *
 *	@langversion ActionScript 3.0
 *	@playerversion Flash 9.0
 *
 *	@author Chaz Hazeltine
 *	@since  15.03.2009
 */
import flash.display.*;
import flash.events.*;
import flash.filters.*;
import flash.geom.*;
import flash.net.*;
import flash.text.*;
import fl.controls.*;

public class NesNtscHarness extends MovieClip {
		
	//--------------------------------------
	// CLASS CONSTANTS
	//--------------------------------------
	[Embed(source="NES_NTSC_window12.pbj", mimeType="application/octet-stream")]
        public static var NESShader12:Class;		
	[Embed(source="NES_NTSC_window8.pbj", mimeType="application/octet-stream")]
		public static var NESShader8:Class;

	//--------------------------------------
	//  CONSTRUCTOR
	//--------------------------------------
	
	/**
	 *	@constructor
	 */
	public function NesNtscHarness()
	{
		super();
		stage.scaleMode = StageScaleMode.NO_SCALE;
		
		// Automatically parse whatever's in the box when we boot up
		_loader = new Loader();
		_loader.contentLoaderInfo.addEventListener( Event.COMPLETE, onLoaderComplete );
		var req:URLRequest = new URLRequest( 'nes_panels.png' );
		_loader.load( req );
	}
	
	//--------------------------------------
	//  PRIVATE VARIABLES
	//--------------------------------------
	private var _shader :Shader;
	private var _filter :ShaderFilter;

	private var _loader :Loader;
	private var _artifacts :Slider;
	private var _fringe :Slider;
	private var _windowSize :Slider;
	private var _bData :BitmapData;
	private var _firstUpdate :Boolean = false;
	private var _isAnim :Boolean = true;
	private var _offset:int = 0;
	
	private var _readout :TextField;
	
	//--------------------------------------
	//  GETTER/SETTERS
	//--------------------------------------
	
	//--------------------------------------
	//  PUBLIC METHODS
	//--------------------------------------
	
	//--------------------------------------
	//  EVENT HANDLERS
	//--------------------------------------
	private function onLoaderComplete( e:Event ) :void {		
		_artifacts = new Slider();
		_fringe = new Slider();
		_windowSize = new Slider();
		var labels:Array = ['Artifacts','Fringing','Window Size'];
		var offsetY:Number = 0;
		for each( var sl:Slider in [_artifacts,_fringe,_windowSize] ) {
			var tf:TextField = new TextField();
			tf.text = labels.shift();
			tf.y = offsetY;
			addChild( tf );
			
			sl.liveDragging = true;
			if( labels.length ) {
				sl.minimum = -5;
				sl.maximum = 5;
				sl.snapInterval = 0.1;
				sl.addEventListener( Event.CHANGE, sliderChange );
			} else {
				// Last slider is the window size
				sl.minimum = 8;
				sl.maximum = 12;
				sl.snapInterval = sl.tickInterval = 4;
				sl.addEventListener( Event.CHANGE, windowSizeChange );
			}
			sl.x = 10;
			sl.y = offsetY + 20;
			addChild( sl );
			
			offsetY += 50;
		}
		
		_readout = new TextField();
		_readout.y = offsetY;
		addChild( _readout );
				
		var spr:Sprite = new Sprite();
		spr.addEventListener( MouseEvent.CLICK, bmpClick );
		spr.x = 100;
		spr.scaleX = spr.scaleY = 2.0;
		addChild( spr );

		_bData = new BitmapData( _loader.width/3, _loader.height );
		var bmp:Bitmap = new Bitmap( _bData );
		spr.addChild( bmp );

		setShader( 8 );

		// Update now
		sliderChange();
		
		addEventListener( Event.ENTER_FRAME, enterFrame );
	}
	
	private function sliderChange( e:Event=null ) :void {
		_shader.data['artifacts'].value = [_artifacts.value];
		_shader.data['fringing'].value = [_fringe.value];		
		updateReadout();
		updateFilter();
	}
	
	private function windowSizeChange( e:Event ) :void {
		setShader( _windowSize.value );
	}
	
	private function bmpClick( e:MouseEvent ) :void {
		var spr:Sprite = Sprite( e.currentTarget );
		spr.scaleX = spr.scaleY = 3.0 - spr.scaleX;
		if( spr.scaleX == 2.0 ) _isAnim = !_isAnim;	// animation: on/off
	}
	
	private function enterFrame( e:Event ) :void {
		if( !_firstUpdate ) {
			_firstUpdate = true;
			_artifacts.value = 1.6;
			_fringe.value = 0.8;
			_windowSize.value = 8;
			sliderChange();
		}
		if( _isAnim ) {
			_offset--;
			if( _offset < -_loader.width ) _offset = 0;
			updateFilter();
		}
	}
		
	//--------------------------------------
	//  PRIVATE & PROTECTED INSTANCE METHODS
	//--------------------------------------
	private function updateReadout() :void {
		_readout.text = ["Artifacts:",_artifacts.value,"\nFringing:",_fringe.value,"\nWindow Size:",_windowSize.value].join(' ');
	}
	
	private function setShader( windowSize:Number ) :void {
		_shader = new Shader( (windowSize==8) ? new NESShader8() : new NESShader12() );
		_shader.precisionHint = ShaderPrecision.FAST;
		_filter = new ShaderFilter(_shader);
		updateReadout();
		updateFilter();
	}
	
	private function updateFilter() :void {
		//_loader.filters = [_filter];	// reapply filter to see the changes
		var mtx:Matrix = new Matrix();
		mtx.translate( _loader.width + _offset, 0 );
		_bData.draw( _loader, mtx );
		mtx.translate( -_loader.width, 0 );
		_bData.draw( _loader, mtx );
		
		var rect:Rectangle = new Rectangle( 0, 0, _loader.width, _loader.height );
		_bData.applyFilter( _bData, rect, new Point(0,0), _filter );
	}
}

}

